# This file is generated from Kodi source code and post-edited
# to correct code style and docstrings formatting.
# License: GPL v.3 <https://www.gnu.org/licenses/gpl-3.0.en.html>
"""
**General functions on Kodi.**

Offers classes and functions that provide information about the media currently
playing and that allow manipulation of the media player (such as starting a new
song). You can also find system information using the functions available in
this library.
"""
from typing import Union, List, Tuple, Optional

__kodistubs__ = True

DRIVE_NOT_READY = 1
ENGLISH_NAME = 2
ISO_639_1 = 0
ISO_639_2 = 1
LOGDEBUG = 0
LOGERROR = 3
LOGFATAL = 4
LOGINFO = 1
LOGNONE = 5
LOGWARNING = 2
PLAYLIST_MUSIC = 0
PLAYLIST_VIDEO = 1
SERVER_AIRPLAYSERVER = 2
SERVER_EVENTSERVER = 6
SERVER_JSONRPCSERVER = 3
SERVER_UPNPRENDERER = 4
SERVER_UPNPSERVER = 5
SERVER_WEBSERVER = 1
SERVER_ZEROCONF = 7
TRAY_CLOSED_MEDIA_PRESENT = 96
TRAY_CLOSED_NO_MEDIA = 64
TRAY_OPEN = 16


class InfoTagMusic:
    """
    **Kodi's music info tag class.**

    To get music info tag data of currently played source.

    .. note::
        Info tag load is only be possible from present player class.

    Example::

        ...
        tag = xbmc.Player().getMusicInfoTag()
        
        title = tag.getTitle()
        url   = tag.getURL()
        ...
    """
    
    def __init__(self) -> None:
        pass
    
    def getDbId(self) -> int:
        """
        Get identification number of tag in database.

        :return: [integer] database id.

        @python_v18 New function added.
        """
        return 0
    
    def getURL(self) -> str:
        """
        Returns url of source as string from music info tag.

        :return: [string] Url of source
        """
        return ""
    
    def getTitle(self) -> str:
        """
        Returns the title from music as string on info tag.

        :return: [string] Music title
        """
        return ""
    
    def getMediaType(self) -> str:
        """
        Get the media type of the music item.

        :return: [string] media type

        Available strings about media type for music:

        ====== ============================= 
        String Description                   
        ====== ============================= 
        artist If it is defined as an artist 
        album  If it is defined as an album  
        song   If it is defined as a song    
        ====== ============================= 

        @python_v18 New function added.
        """
        return ""
    
    def getArtist(self) -> str:
        """
        Returns the artist from music as string if present.

        :return: [string] Music artist
        """
        return ""
    
    def getAlbum(self) -> str:
        """
        Returns the album from music tag as string if present.

        :return: [string] Music album name
        """
        return ""
    
    def getAlbumArtist(self) -> str:
        """
        Returns the album artist from music tag as string if present.

        :return: [string] Music album artist name
        """
        return ""
    
    def getGenre(self) -> str:
        """
        Returns the genre name from music tag as string if present.

        :return: [string] Genre name
        """
        return ""
    
    def getDuration(self) -> int:
        """
        Returns the duration of music as integer from info tag.

        :return: [integer] Duration
        """
        return 0
    
    def getRating(self) -> int:
        """
        Returns the scraped rating as integer.

        :return: [integer] Rating
        """
        return 0
    
    def getUserRating(self) -> int:
        """
        Returns the user rating as integer (-1 if not existing)

        :return: [integer] User rating
        """
        return 0
    
    def getTrack(self) -> int:
        """
        Returns the track number (if present) from music info tag as integer.

        :return: [integer] Track number
        """
        return 0
    
    def getDisc(self) -> int:
        """
        Returns the disk number (if present) from music info tag as integer.

        :return: [integer] Disc number
        """
        return 0
    
    def getReleaseDate(self) -> str:
        """
        Returns the release date as string from music info tag (if present).

        :return: [string] Release date
        """
        return ""
    
    def getListeners(self) -> int:
        """
        Returns the listeners as integer from music info tag.

        :return: [integer] Listeners
        """
        return 0
    
    def getPlayCount(self) -> int:
        """
        Returns the number of carried out playbacks.

        :return: [integer] Playback count
        """
        return 0
    
    def getLastPlayed(self) -> str:
        """
        Returns last played time as string from music info tag.

        :return: [string] Last played date / time on tag
        """
        return ""
    
    def getComment(self) -> str:
        """
        Returns comment as string from music info tag.

        :return: [string] Comment on tag
        """
        return ""
    
    def getLyrics(self) -> str:
        """
        Returns a string from lyrics.

        :return: [string] Lyrics on tag
        """
        return ""
    
    def getMusicBrainzTrackID(self) -> str:
        """
        Returns the MusicBrainz Recording ID from music info tag (if present).

        :return: [string] MusicBrainz Recording ID

        @python_v19 New function added.
        """
        return ""
    
    def getMusicBrainzArtistID(self) -> List[str]:
        """
        Returns the MusicBrainz Artist IDs from music info tag (if present).

        :return: [list] MusicBrainz Artist IDs

        @python_v19 New function added.
        """
        return [""]
    
    def getMusicBrainzAlbumID(self) -> str:
        """
        Returns the MusicBrainz Release ID from music info tag (if present).

        :return: [string] MusicBrainz Release ID

        @python_v19 New function added.
        """
        return ""
    
    def getMusicBrainzReleaseGroupID(self) -> str:
        """
        Returns the MusicBrainz Release Group ID from music info tag (if present).

        :return: [string] MusicBrainz Release Group ID

        @python_v19 New function added.
        """
        return ""
    
    def getMusicBrainzAlbumArtistID(self) -> List[str]:
        """
        Returns the MusicBrainz Release Artist IDs from music info tag (if present).

        :return: [list] MusicBrainz Release Artist IDs

        @python_v19 New function added.
        """
        return [""]
    

class InfoTagRadioRDS:
    """
    **Kodi's radio RDS info tag class.**

    To get radio RDS info tag data of currently played `PVR` radio channel source.

    .. note::
        Info tag load is only be possible from present player class.  Also
        is all the data variable from radio channels and not known on
        beginning of radio receiving.

    Example::

        ...
        tag = xbmc.Player().getRadioRDSInfoTag()
        
        title  = tag.getTitle()
        artist = tag.getArtist()
        ...
    """
    
    def __init__(self) -> None:
        pass
    
    def getTitle(self) -> str:
        """
        Title of the item on the air; i.e. song title.

        :return: Title
        """
        return ""
    
    def getBand(self) -> str:
        """
        Band of the item on air.

        :return: Band
        """
        return ""
    
    def getArtist(self) -> str:
        """
        Artist of the item on air.

        :return: Artist
        """
        return ""
    
    def getComposer(self) -> str:
        """
        Get the Composer of the music.

        :return: Composer
        """
        return ""
    
    def getConductor(self) -> str:
        """
        Get the Conductor of the Band.

        :return: Conductor
        """
        return ""
    
    def getAlbum(self) -> str:
        """
        Album of item on air.

        :return: Album name
        """
        return ""
    
    def getComment(self) -> str:
        """
        Get Comment text from channel.

        :return: Comment
        """
        return ""
    
    def getAlbumTrackNumber(self) -> int:
        """
        Get the album track number of currently sended music.

        :return: Track Number
        """
        return 0
    
    def getInfoNews(self) -> str:
        """
        Get News informations.

        :return: News Information
        """
        return ""
    
    def getInfoNewsLocal(self) -> str:
        """
        Get Local news informations.

        :return: Local News Information
        """
        return ""
    
    def getInfoSport(self) -> str:
        """
        Get Sport informations.

        :return: Sport Information
        """
        return ""
    
    def getInfoStock(self) -> str:
        """
        Get Stock informations.

        :return: Stock Information
        """
        return ""
    
    def getInfoWeather(self) -> str:
        """
        Get Weather informations.

        :return: Weather Information
        """
        return ""
    
    def getInfoHoroscope(self) -> str:
        """
        Get Horoscope informations.

        :return: Horoscope Information
        """
        return ""
    
    def getInfoCinema(self) -> str:
        """
        Get Cinema informations.

        :return: Cinema Information
        """
        return ""
    
    def getInfoLottery(self) -> str:
        """
        Get Lottery informations.

        :return: Lottery Information
        """
        return ""
    
    def getInfoOther(self) -> str:
        """
        Get other informations.

        :return: Other Information
        """
        return ""
    
    def getEditorialStaff(self) -> str:
        """
        Get Editorial Staff names.

        :return: Editorial Staff
        """
        return ""
    
    def getProgStation(self) -> str:
        """
        Name describing station.

        :return: Program Station
        """
        return ""
    
    def getProgStyle(self) -> str:
        """
        The the radio channel style currently used.

        :return: Program Style
        """
        return ""
    
    def getProgHost(self) -> str:
        """
        Host of current radio show.

        :return: Program Host
        """
        return ""
    
    def getProgWebsite(self) -> str:
        """
        Link to URL (web page) for radio station homepage.

        :return: Program Website
        """
        return ""
    
    def getProgNow(self) -> str:
        """
        Current radio program show.

        :return: Program Now
        """
        return ""
    
    def getProgNext(self) -> str:
        """
        Next program show.

        :return: Program Next
        """
        return ""
    
    def getPhoneHotline(self) -> str:
        """
        Telephone number of the radio station's hotline.

        :return: Phone Hotline
        """
        return ""
    
    def getEMailHotline(self) -> str:
        """
        Email address of the radio station's studio.

        :return: EMail Hotline
        """
        return ""
    
    def getPhoneStudio(self) -> str:
        """
        Telephone number of the radio station's studio.

        :return: Phone Studio
        """
        return ""
    
    def getEMailStudio(self) -> str:
        """
        Email address of radio station studio.

        :return: EMail Studio
        """
        return ""
    
    def getSMSStudio(self) -> str:
        """
        SMS (Text Messaging) number for studio.

        :return: SMS Studio
        """
        return ""
    

class InfoTagVideo:
    """
    **Kodi's video info tag class.**

    To get video info tag data of currently played source.

    .. note::
        Info tag load is only be possible from present player class.

    Example::

        ...
        tag = xbmc.Player().getVideoInfoTag()
        
        title = tag.getTitle()
        file  = tag.getFile()
        ...
    """
    
    def __init__(self) -> None:
        pass
    
    def getDbId(self) -> int:
        """
        Get identification number of tag in database

        :return: [integer] database id

        @python_v17 New function added.
        """
        return 0
    
    def getDirector(self) -> str:
        """
        Getfilm director who has made the film (if present).

        :return: [string] Film director name.
        """
        return ""
    
    def getWritingCredits(self) -> str:
        """
        Get the writing credits if present from video info tag.

        :return: [string] Writing credits
        """
        return ""
    
    def getGenre(self) -> str:
        """
        To get theVideo Genre if available.

        :return: [string] Genre name
        """
        return ""
    
    def getTagLine(self) -> str:
        """
        Get video tag line if available.

        :return: [string] Video tag line
        """
        return ""
    
    def getPlotOutline(self) -> str:
        """
        Get the outline plot of the video if present.

        :return: [string] Outline plot
        """
        return ""
    
    def getPlot(self) -> str:
        """
        Get the plot of the video if present.

        :return: [string] Plot
        """
        return ""
    
    def getPictureURL(self) -> str:
        """
        Get a picture URL of the video to show as screenshot.

        :return: [string] Picture URL
        """
        return ""
    
    def getTitle(self) -> str:
        """
        Get the video title.

        :return: [string] Video title
        """
        return ""
    
    def getTVShowTitle(self) -> str:
        """
        Get the video TV show title.

        :return: [string] TV show title

        @python_v17 New function added.
        """
        return ""
    
    def getMediaType(self) -> str:
        """
        Get the media type of the video.

        :return: [string] media type

        Available strings about media type for video:

        ========== ==================================== 
        String     Description                          
        ========== ==================================== 
        video      For normal video                     
        set        For a selection of video             
        musicvideo To define it as music video          
        movie      To define it as normal movie         
        tvshow     If this is it defined as tvshow      
        season     The type is used as a series season  
        episode    The type is used as a series episode 
        ========== ==================================== 

        @python_v17 New function added.
        """
        return ""
    
    def getVotes(self) -> str:
        """
        Get the video votes if available from video info tag.

        :return: [string] Votes
        """
        return ""
    
    def getCast(self) -> str:
        """
        To get the cast of the video when available.

        :return: [string] Video casts
        """
        return ""
    
    def getFile(self) -> str:
        """
        To get the video file name.

        :return: [string] File name
        """
        return ""
    
    def getPath(self) -> str:
        """
        To get the path where the video is stored.

        :return: [string] Path
        """
        return ""
    
    def getFilenameAndPath(self) -> str:
        """
        To get the full path with filename where the video is stored.

        :return: [string] File name and Path

        @python_v19 New function added.
        """
        return ""
    
    def getIMDBNumber(self) -> str:
        """
        To get theIMDb number of the video (if present).

        :return: [string] IMDb number
        """
        return ""
    
    def getSeason(self) -> int:
        """
        To get season number of a series

        :return: [integer] season number

        @python_v17 New function added.
        """
        return 0
    
    def getEpisode(self) -> int:
        """
        To get episode number of a series

        :return: [integer] episode number

        @python_v17 New function added.
        """
        return 0
    
    def getYear(self) -> int:
        """
        Get production year of video if present.

        :return: [integer] Production Year
        """
        return 0
    
    def getRating(self) -> float:
        """
        Get the video rating if present as float (double where supported).

        :return: [float] The rating of the video
        """
        return 0.0
    
    def getUserRating(self) -> int:
        """
        Get the user rating if present as integer.

        :return: [integer] The user rating of the video
        """
        return 0
    
    def getPlayCount(self) -> int:
        """
        To get the number of plays of the video.

        :return: [integer] Play Count
        """
        return 0
    
    def getLastPlayed(self) -> str:
        """
        Get the last played date / time as string.

        :return: [string] Last played date / time
        """
        return ""
    
    def getOriginalTitle(self) -> str:
        """
        To get the original title of the video.

        :return: [string] Original title
        """
        return ""
    
    def getPremiered(self) -> str:
        """
        To getpremiered date of the video, if available.

        :return: [string]
        """
        return ""
    
    def getFirstAired(self) -> str:
        """
        Returns first aired date as string from info tag.

        :return: [string] First aired date
        """
        return ""
    
    def getTrailer(self) -> str:
        """
        To get the path where the trailer is stored.

        :return: [string] Trailer path

        @python_v17 New function added.
        """
        return ""
    
    def getArtist(self) -> List[str]:
        """
        To get the artist name (for musicvideos)

        :return: [std::vector<std::string>] Artist name

        @python_v18 New function added.
        """
        return [""]
    
    def getAlbum(self) -> str:
        """
        To get the album name (for musicvideos)

        :return: [string] Album name

        @python_v18 New function added.
        """
        return ""
    
    def getTrack(self) -> int:
        """
        To get the track number (for musicvideos)

        :return: [int] Track number

        @python_v18 New function added.
        """
        return 0
    
    def getDuration(self) -> int:
        """
        To get the duration

        :return: [unsigned int] Duration

        @python_v18 New function added.
        """
        return 0
    

class Keyboard:
    """
    **Kodi's keyboard class.**

    Creates a new `Keyboard` object with default text heading and hidden input flag
    if supplied.

    :param default: : [opt] string - default text entry.
    :param heading: : [opt] string - keyboard heading.
    :param hidden: : [opt] boolean - True for hidden text entry.

    Example::

        ..
        kb = xbmc.Keyboard('default', 'heading', True)
        kb.setDefault('password') # optional
        kb.setHeading('Enter password') # optional
        kb.setHiddenInput(True) # optional
        kb.doModal()
        if kb.isConfirmed():
            text = kb.getText()
        ..
    """
    
    def __init__(self, line: str = "",
                 heading: str = "",
                 hidden: bool = False) -> None:
        pass
    
    def doModal(self, autoclose: int = 0) -> None:
        """
        Show keyboard and wait for user action.

        :param autoclose: [opt] integer - milliseconds to autoclose dialog.
            (default=do not autoclose)

        Example::

            ..
            kb.doModal(30000)
            ..
        """
        pass
    
    def setDefault(self, line: str = "") -> None:
        """
        Set the default text entry.

        :param line: string - default text entry.

        Example::

            ..
            kb.setDefault('password')
            ..
        """
        pass
    
    def setHiddenInput(self, hidden: bool = False) -> None:
        """
        Allows hidden text entry.

        :param hidden: boolean - True for hidden text entry.

        Example::

            ..
            kb.setHiddenInput(True)
            ..
        """
        pass
    
    def setHeading(self, heading: str) -> None:
        """
        Set the keyboard heading.

        :param heading: string - keyboard heading.

        Example::

            ..
            kb.setHeading('Enter password')
            ..
        """
        pass
    
    def getText(self) -> str:
        """
        Returns the user input as a string.

        .. note::
            This will always return the text entry even if you cancel the keyboard.
            Use the `isConfirmed()` method to check if user cancelled the keyboard.

        :return: get the in keyboard entered text

        Example::

            ..
            text = kb.getText()
            ..
        """
        return ""
    
    def isConfirmed(self) -> bool:
        """
        Returns False if the user cancelled the input.

        :return: true if confirmed, if cancelled false

        Example::

            ..
            if (kb.isConfirmed()):
            ..
        """
        return True
    

class Monitor:
    """
    **Kodi's monitor class.**

    Creates a new monitor to notify addon about changes.
    """
    
    def __init__(self) -> None:
        pass
    
    def onSettingsChanged(self) -> None:
        """
        onSettingsChanged method.

        Will be called when addon settings are changed
        """
        pass
    
    def onScreensaverActivated(self) -> None:
        """
        onScreensaverActivated method.

        Will be called when screensaver kicks in
        """
        pass
    
    def onScreensaverDeactivated(self) -> None:
        """
        onScreensaverDeactivated method.

        Will be called when screensaver goes off
        """
        pass
    
    def onDPMSActivated(self) -> None:
        """
        onDPMSActivated method.

        Will be called when energysaving/DPMS gets active
        """
        pass
    
    def onDPMSDeactivated(self) -> None:
        """
        onDPMSDeactivated method.

        Will be called when energysaving/DPMS is turned off
        """
        pass
    
    def onScanStarted(self, library: str) -> None:
        """
        onScanStarted method.

        :param library: Video / music as string

        .. note::
            Will be called when library clean has ended and return video or
            music to indicate which library is being scanned

        @python_v14 New function added.
        """
        pass
    
    def onScanFinished(self, library: str) -> None:
        """
        onScanFinished method.

        :param library: Video / music as string

        .. note::
            Will be called when library clean has ended and return video or
            music to indicate which library has been scanned

        @python_v14 New function added.
        """
        pass
    
    def onCleanStarted(self, library: str) -> None:
        """
        onCleanStarted method.

        :param library: Video / music as string

        .. note::
            Will be called when library clean has ended and return video or
            music to indicate which library has been cleaned

        @python_v14 New function added.
        """
        pass
    
    def onCleanFinished(self, library: str) -> None:
        """
        onCleanFinished method.

        :param library: Video / music as string

        .. note::
            Will be called when library clean has ended and return video or
            music to indicate which library has been finished

        @python_v14 New function added.
        """
        pass
    
    def onNotification(self, sender: str, method: str, data: str) -> None:
        """
        onNotification method.

        :param sender: Sender of the notification
        :param method: Name of the notification
        :param data: JSON-encoded data of the notification

        .. note::
            Will be called when Kodi receives or sends a notification

        @python_v13 New function added.
        """
        pass
    
    def waitForAbort(self, timeout: float = -1) -> bool:
        """
        Wait for Abort

        Block until abort is requested, or until timeout occurs. If an abort requested
        have already been made, return immediately.

        :param timeout: [opt] float - timeout in seconds. Default: no timeout.
        :return: True when abort have been requested, False if a timeout is given and the operation times out.

        @python_v14 New function added.

        Example::

            ..
            monitor = xbmc.Monitor()
            # do something
            monitor.waitForAbort(10) # sleeps for 10 secs or returns early if kodi aborts
            if monitor.abortRequested():
            # abort was requested to Kodi (e.g. shutdown), do your cleanup logic
            ..
        """
        return True
    
    def abortRequested(self) -> bool:
        """
        Returns True if abort has been requested.

        :return: True if requested

        @python_v14 New function added.
        """
        return True
    

class Player:
    """
    **Kodi's player.**

    To become and create the class to play something.

    Example::

        ...
        xbmc.Player().play(url, listitem, windowed)
        ...
    """
    
    def __init__(self) -> None:
        pass
    
    def play(self, item: Union[str,  'PlayList'] = "",
             listitem: Optional['xbmcgui.ListItem'] = None,
             windowed: bool = False,
             startpos: int = -1) -> None:
        """
        Play an item.

        :param item: [opt] string - filename, url or playlist
        :param listitem: [opt] listitem - used with setInfo() to set different infolabels.
        :param windowed: [opt] bool - true=play video windowed, false=play users
            preference. (default)
        :param startpos: [opt] int - starting position when playing a playlist. Default = -1

        .. note::
            If item is not given then the `Player` will try to play the current
            item in the current playlist. You can use the above as keywords
            for arguments and skip certain optional arguments.  Once you use a
            keyword, all following arguments require the keyword.

        Example::

            ...
            listitem = xbmcgui.ListItem('Ironman')
            listitem.setInfo('video', {'Title': 'Ironman', 'Genre': 'Science Fiction'})
            xbmc.Player().play(url, listitem, windowed)
            xbmc.Player().play(playlist, listitem, windowed, startpos)
            ...
        """
        pass
    
    def stop(self) -> None:
        """
        Stop playing.
        """
        pass
    
    def pause(self) -> None:
        """
        Pause or resume playing if already paused.
        """
        pass
    
    def playnext(self) -> None:
        """
        Play next item in playlist.
        """
        pass
    
    def playprevious(self) -> None:
        """
        Play previous item in playlist.
        """
        pass
    
    def playselected(self, selected: int) -> None:
        """
        Play a certain item from the current playlist.

        :param selected: Integer - Item to select
        """
        pass
    
    def isPlaying(self) -> bool:
        """
        Check Kodi is playing something.

        :return: True if Kodi is playing a file.
        """
        return True
    
    def isPlayingAudio(self) -> bool:
        """
        Check for playing audio.

        :return: True if Kodi is playing an audio file.
        """
        return True
    
    def isPlayingVideo(self) -> bool:
        """
        Check for playing video.

        :return: True if Kodi is playing a video.
        """
        return True
    
    def isPlayingRDS(self) -> bool:
        """
        Check for playing radio data system (RDS).

        :return: True if kodi is playing a radio data system (RDS).
        """
        return True
    
    def isExternalPlayer(self) -> bool:
        """
        Check for external player.

        :return: True if kodi is playing using an external player.

        @python_v18 New function added.
        """
        return True
    
    def getPlayingFile(self) -> str:
        """
        Returns the current playing file as a string.

        .. note::
            For LiveTV, returns a **pvr://** url which is not translatable to
            an OS specific file or external url.

        :return: Playing filename
        :raises Exception: If player is not playing a file.
        """
        return ""
    
    def getTime(self) -> float:
        """
        Get playing time.

        Returns the current time of the current playing media as fractional seconds.

        :return: Current time as fractional seconds
        :raises Exception: If player is not playing a file.
        """
        return 0.0
    
    def seekTime(self, seekTime: float) -> None:
        """
        Seek time.

        Seeks the specified amount of time as fractional seconds. The time specified is
        relative to the beginning of the currently. playing media file.

        :param seekTime: Time to seek as fractional seconds
        :raises Exception: If player is not playing a file.
        """
        pass
    
    def setSubtitles(self, subtitleFile: str) -> None:
        """
        Set subtitle file and enable subtitles.

        :param subtitleFile: File to use as source ofsubtitles
        """
        pass
    
    def showSubtitles(self, bVisible: bool) -> None:
        """
        Enable / disable subtitles.

        :param visible: [boolean] True for visible subtitles.

        Example::

            ...
            xbmc.Player().showSubtitles(True)
            ...
        """
        pass
    
    def getSubtitles(self) -> str:
        """
        Get subtitle stream name.

        :return: Stream name
        """
        return ""
    
    def getAvailableSubtitleStreams(self) -> List[str]:
        """
        Get Subtitle stream names.

        :return: `List` of subtitle streams as name
        """
        return [""]
    
    def setSubtitleStream(self, iStream: int) -> None:
        """
        Set Subtitle Stream.

        :param iStream: [int] Subtitle stream to select for play

        Example::

            ...
            xbmc.Player().setSubtitleStream(1)
            ...
        """
        pass
    
    def updateInfoTag(self, item: 'xbmcgui.ListItem') -> None:
        """
        Update info labels for currently playing item.

        :param item: ListItem with new info
        :raises Exception: If player is not playing a file

        @python_v18 New function added.

        Example::

            ...
            item = xbmcgui.ListItem()
            item.setPath(xbmc.Player().getPlayingFile())
            item.setInfo('music', {'title' : 'foo', 'artist' : 'bar'})
            xbmc.Player().updateInfoTag(item)
            ...
        """
        pass
    
    def getVideoInfoTag(self) -> InfoTagVideo:
        """
        To get video info tag.

        Returns the VideoInfoTag of the current playing Movie.

        :return: Video info tag
        :raises Exception: If player is not playing a file or current file is not a movie file.
        """
        return InfoTagVideo()
    
    def getMusicInfoTag(self) -> InfoTagMusic:
        """
        To get music info tag.

        Returns the MusicInfoTag of the current playing 'Song'.

        :return: Music info tag
        :raises Exception: If player is not playing a file or current file is not a music file.
        """
        return InfoTagMusic()
    
    def getRadioRDSInfoTag(self) -> InfoTagRadioRDS:
        """
        To get Radio RDS info tag

        Returns the RadioRDSInfoTag of the current playing 'Radio Song if. present'.

        :return: Radio RDS info tag
        :raises Exception: If player is not playing a file or current file is not a rds file.
        """
        return InfoTagRadioRDS()
    
    def getTotalTime(self) -> float:
        """
        To get total playing time.

        Returns the total time of the current playing media in seconds. This is only
        accurate to the full second.

        :return: Total time of the current playing media
        :raises Exception: If player is not playing a file.
        """
        return 0.0
    
    def getAvailableAudioStreams(self) -> List[str]:
        """
        Get Audio stream names

        :return: List of audio streams as name
        """
        return [""]
    
    def setAudioStream(self, iStream: int) -> None:
        """
        Set Audio Stream.

        :param iStream: [int] Audio stream to select for play

        Example::

            ...
            xbmc.Player().setAudioStream(1)
            ...
        """
        pass
    
    def getAvailableVideoStreams(self) -> List[str]:
        """
        Get Video stream names

        :return: List of video streams as name
        """
        return [""]
    
    def setVideoStream(self, iStream: int) -> None:
        """
        Set Video Stream.

        :param iStream: [int] Video stream to select for play

        Example::

            ...
            xbmc.Player().setVideoStream(1)
            ...
        """
        pass
    
    def onPlayBackStarted(self) -> None:
        """
        onPlayBackStarted method.

        Will be called when Kodi player starts. Video or audio might not be available at
        this point.

        @python_v18 Use `onAVStarted()` instead if you need to detect if Kodi is actually
        playing a media file (i.e, if a stream is available)
        """
        pass
    
    def onAVStarted(self) -> None:
        """
        onAVStarted method.

        Will be called when Kodi has a video or audiostream.

        @python_v18 New function added.
        """
        pass
    
    def onAVChange(self) -> None:
        """
        onAVChange method.

        Will be called when Kodi has a video, audio or subtitle stream. Also happens
        when the stream changes.

        @python_v18 New function added.
        """
        pass
    
    def onPlayBackEnded(self) -> None:
        """
        onPlayBackEnded method.

        Will be called when Kodi stops playing a file.
        """
        pass
    
    def onPlayBackStopped(self) -> None:
        """
        onPlayBackStopped method.

        Will be called when user stops Kodi playing a file.
        """
        pass
    
    def onPlayBackError(self) -> None:
        """
        onPlayBackError method.

        Will be called when playback stops due to an error.
        """
        pass
    
    def onPlayBackPaused(self) -> None:
        """
        onPlayBackPaused method.

        Will be called when user pauses a playing file.
        """
        pass
    
    def onPlayBackResumed(self) -> None:
        """
        onPlayBackResumed method.

        Will be called when user resumes a paused file.
        """
        pass
    
    def onQueueNextItem(self) -> None:
        """
        onQueueNextItem method.

        Will be called when user queues the next item.
        """
        pass
    
    def onPlayBackSpeedChanged(self, speed: int) -> None:
        """
        onPlayBackSpeedChanged method.

        Will be called when players speed changes (eg. user FF/RW).

        :param speed: [integer] Current speed of player

        .. note::
            Negative speed means player is rewinding, 1 is normal playback
            speed.
        """
        pass
    
    def onPlayBackSeek(self, time: int, seekOffset: int) -> None:
        """
        onPlayBackSeek method.

        Will be called when user seeks to a time.

        :param time: [integer] Time to seek to
        :param seekOffset: [integer] ?
        """
        pass
    
    def onPlayBackSeekChapter(self, chapter: int) -> None:
        """
        onPlayBackSeekChapter method.

        Will be called when user performs a chapter seek.

        :param chapter: [integer] Chapter to seek to
        """
        pass
    

class PlayList:
    """
    **Kodi's Play `List` class.**

    To create and edit a playlist which can be handled by the player.

    :param playList: [integer] To define the stream type

    ===== =================== =================================== 
    Value Integer String      Description                         
    ===== =================== =================================== 
    0     xbmc.PLAYLIST_MUSIC Playlist for music files or streams 
    1     xbmc.PLAYLIST_VIDEO Playlist for video files or streams 
    ===== =================== =================================== 

    Example::

        ...
        play=xbmc.PlayList(xbmc.PLAYLIST_VIDEO)
        ...
    """
    
    def __init__(self, playList: int) -> None:
        pass
    
    def getPlayListId(self) -> int:
        """
        Get the `PlayList` Identifier

        :return: Id as an integer.
        """
        return 0
    
    def add(self, url: str,
            listitem: Optional['xbmcgui.ListItem'] = None,
            index: int = -1) -> None:
        """
        Adds a new file to the playlist.

        :param url: string or unicode - filename or url to add.
        :param listitem: [opt] listitem - used with setInfo() to set different infolabels.
        :param index: [opt] integer - position to add playlist item. (default=end)

        .. note::
            You can use the above as keywords for arguments and skip certain
            optional arguments. Once you use a keyword, all following
            arguments require the keyword.

        Example::

            ..
            playlist = xbmc.PlayList(xbmc.PLAYLIST_VIDEO)
            video = 'F:\\movies\\Ironman.mov'
            listitem = xbmcgui.ListItem('Ironman', thumbnailImage='F:\\movies\\Ironman.tbn')
            listitem.setInfo('video', {'Title': 'Ironman', 'Genre': 'Science Fiction'})
            playlist.add(url=video, listitem=listitem, index=7)
            ..
        """
        pass
    
    def load(self, filename: str) -> bool:
        """
        Load a playlist.

        Clear current playlist and copy items from the file to this Playlist filename
        can be like .pls or .m3u ...

        :param filename: File with list to play inside
        :return: False if unable to load playlist
        """
        return True
    
    def remove(self, filename: str) -> None:
        """
        Remove an item with this filename from the playlist.

        :param filename: The file to remove from list.
        """
        pass
    
    def clear(self) -> None:
        """
        Clear all items in the playlist.
        """
        pass
    
    def size(self) -> int:
        """
        Returns the total number of PlayListItems in this playlist.

        :return: Amount of playlist entries.
        """
        return 0
    
    def shuffle(self) -> None:
        """
        Shuffle the playlist.
        """
        pass
    
    def unshuffle(self) -> None:
        """
        Unshuffle the playlist.
        """
        pass
    
    def getposition(self) -> int:
        """
        Returns the position of the current song in this playlist.

        :return: Position of the current song
        """
        return 0
    

class RenderCapture:
    """
    **Kodi's render capture.**
    """
    
    def __init__(self) -> None:
        pass
    
    def getWidth(self) -> int:
        """
        Get width

        To get width of captured image as set during `RenderCapture.capture()`. Returns 0
        prior to calling capture.

        :return: Width or 0 prior to calling capture
        """
        return 0
    
    def getHeight(self) -> int:
        """
        Get height

        To get height of captured image as set during `RenderCapture.capture()`. Returns
        0 prior to calling capture.

        :return: height or 0 prior to calling capture
        """
        return 0
    
    def getAspectRatio(self) -> float:
        """
        Get aspect ratio of currently displayed video.

        :return: Aspect ratio

        This may be called prior to calling `RenderCapture.capture()`.
        """
        return 0.0
    
    def getImageFormat(self) -> str:
        """
        Get image format

        :return: Format of captured image: 'BGRA'

        @python_v17 Image will now always be returned in BGRA
        """
        return ""
    
    def getImage(self, msecs: int = 0) -> bytearray:
        """
        Returns captured image as a bytearray.

        :param msecs: [opt] Milliseconds to wait. Waits 1000ms if not specified
        :return: Captured image as a bytearray

        .. note::
            The size of the image is m_width * m_height * 4

        @python_v17 Added the option to specify wait time in msec.
        """
        return bytearray()
    
    def capture(self, width: int, height: int) -> None:
        """
        Issue capture request.

        :param width: Width capture image should be rendered to
        :param height: Height capture image should should be rendered to

        @python_v17 Removed the option to pass **flags**
        """
        pass


def log(msg: str, level: int = LOGDEBUG) -> None:
    """
    Write a string to Kodi's log file and the debug window.

    :param msg: string - text to output.
    :param level: [opt] integer - log level to output at.(default=LOGDEBUG)

    =============== ================================================================================
    Value:          Description:
    =============== ================================================================================
    xbmc.LOGDEBUG   In depth information about the status of Kodi. This information can pretty much
                    only be deciphered by a developer or long time Kodi power user.
    xbmc.LOGINFO    Something has happened. It's not a problem, we just thought you might want
                    to know. Fairly excessive output that most people won't care about.
    xbmc.LOGWARNING Something potentially bad has happened. If Kodi did something you didn't expect,
                    this is probably why. Watch for errors to follow.
    xbmc.LOGERROR   This event is bad. Something has failed. You likely noticed problems with
                    the application be it skin artifacts, failure of playback a crash, etc.
    xbmc.LOGFATAL   We're screwed. Kodi is about to crash.
    =============== ================================================================================

    .. note::
        Addon developers are advised to keep ``LOGDEBUG`` as the default
        logging level and to use conservative logging (log only if
        needed). Excessive logging makes it harder to debug kodi itself.

    Logging in kodi has a global configuration level that controls how
    text is written to the log. This global logging behaviour can be
    changed in the GUI (**Settings -> System -> Logging**) (debug toggle)
    or furthered configured in advancedsettings (loglevel setting).
    Text is written to the log for the following conditions:
    loglevel == -1 (NONE, nothing at all is logged to the log)

    loglevel == 0 (NORMAL, shows ``LOGINFO``,``LOGWARNING``,``LOGERROR``
    and ``LOGFATAL``) - Default kodi behaviour

    loglevel == 1 (DEBUG, shows all) - Behaviour if you toggle debug log in the GUI

    @python_v17 Default level changed from ``LOGNOTICE`` to ``LOGDEBUG``

    @python_v19 Removed ``LOGNOTICE`` (use ``LOGINFO``) and ``LOGSEVERE`` (use ``LOGFATAL``)

    Example::

        ..
        xbmc.log(msg='This is a test string.', level=xbmc.LOGDEBUG);
        ..
    """
    pass


def shutdown() -> None:
    """
    Shutdown the htpc.

    Example::

        ..
        xbmc.shutdown()
        ..
    """
    pass


def restart() -> None:
    """
    Restart the htpc.

    Example::

        ..
        xbmc.restart()
        ..
    """
    pass


def executescript(script: str) -> None:
    """
    Execute a python script.

    :param script: string - script filename to execute.

    Example::

        ..
        xbmc.executescript('special://home/scripts/update.py')
        ..
    """
    pass


def executebuiltin(function: str, wait: bool = False) -> None:
    """
    Execute a built in Kodi function.

    :param function: string - builtin function to execute.

    List of builtin functions

    Example::

        ..
        xbmc.executebuiltin('Skin.SetString(abc,def)')
        ..
    """
    pass


def executeJSONRPC(jsonrpccommand: str) -> str:
    """
    Execute an JSONRPC command.

    :param jsonrpccommand: string - jsonrpc command to execute.
    :return: jsonrpc return string

    Example::

        ..
        response = xbmc.executeJSONRPC('{ "jsonrpc": "2.0", "method": "JSONRPC.Introspect", "id": 1 }')
        ..
    """
    return ""


def sleep(timemillis: int) -> None:
    """
    Sleeps for 'time' (msec).

    :param time: integer - number of msec to sleep.
    :raises TypeError: If time is not an integer.

    This is useful if you need to sleep for a small amount of time (milisecond
    range) somewhere in your addon logic. Please note that Kodi will attempt to stop
    any running scripts when signaled to exit and wait for a maximum of 5 seconds
    before trying to force stop your script. If your addon makes use
    of `xbmc.sleep()` incorrectly (long periods of time, e.g. that exceed the force
    stop waiting time) it may lead to Kodi hanging on shutdown. In case your addon
    needs long sleep/idle periods use `xbmc.Monitor().waitForAbort(secs)` instead.

    Example::

        ..
        xbmc.sleep(2000) # sleeps for 2 seconds
        ..
    """
    pass


def getLocalizedString(id: int) -> str:
    """
    Get a localized 'unicode string'.

    :param id: integer - id# for string you want to localize.
    :return: Localized 'unicode string'

    .. note::
        See strings.po in``\language\{yourlanguage}\`` for which id you
        need for a string.

    Example::

        ..
        locstr = xbmc.getLocalizedString(6)
        ..
    """
    return ""


def getSkinDir() -> str:
    """
    Get the active skin directory.

    :return: The active skin directory as a string

    .. note::
        This is not the full path like
        'special://home/addons/MediaCenter', but only 'MediaCenter'.

    Example::

        ..
        skindir = xbmc.getSkinDir()
        ..
    """
    return ""


def getLanguage(format: int = ENGLISH_NAME, region: bool = False) -> str:
    """
    Get the active language.

    :param format: [opt] format of the returned language string

    ================= ========================================================== 
    Value             Description                                                
    ================= ========================================================== 
    xbmc.ISO_639_1    Two letter code as defined in ISO 639-1                    
    xbmc.ISO_639_2    Three letter code as defined in ISO 639-2/T or ISO 639-2/B 
    xbmc.ENGLISH_NAME Full language name in English (default)                    
    ================= ========================================================== 

    :param region: [opt] append the region delimited by "-" of the language (setting) to
        the returned language string
    :return: The active language as a string

    @python_v13 Added new options **format** and **region**.

    Example::

        ..
        language = xbmc.getLanguage(xbmc.ENGLISH_NAME)
        ..
    """
    return ""


def getIPAddress() -> str:
    """
    Get the current ip address.

    :return: The current ip address as a string

    Example::

        ..
        ip = xbmc.getIPAddress()
        ..
    """
    return ""


def getDVDState() -> int:
    """
    Returns the dvd state as an integer.

    :return: Values for state are:

    ===== ============================== 
    Value Name                           
    ===== ============================== 
    1     xbmc.DRIVE_NOT_READY           
    16    xbmc.TRAY_OPEN                 
    64    xbmc.TRAY_CLOSED_NO_MEDIA      
    96    xbmc.TRAY_CLOSED_MEDIA_PRESENT 
    ===== ============================== 

    Example::

        ..
        dvdstate = xbmc.getDVDState()
        ..
    """
    return 0


def getFreeMem() -> int:
    """
    Get amount of free memory in MB.

    :return: The amount of free memory in MB as an integer

    Example::

        ..
        freemem = xbmc.getFreeMem()
        ..
    """
    return 0


def getInfoLabel(cLine: str) -> str:
    """
    Get a info label

    :param infotag: string - infoTag for value you want returned.
    :return: InfoLabel as a string

    List of InfoTags - http://kodi.wiki/view/InfoLabels

    Example::

        ..
        label = xbmc.getInfoLabel('Weather.Conditions')
        ..
    """
    return ""


def getInfoImage(infotag: str) -> str:
    """
    Get filename including path to the InfoImage's thumbnail.

    :param infotag: string - infotag for value you want returned
    :return: Filename including path to the InfoImage's thumbnail as a string

    List of InfoTags - http://kodi.wiki/view/InfoLabels

    Example::

        ..
        filename = xbmc.getInfoImage('Weather.Conditions')
        ..
    """
    return ""


def playSFX(filename: str, useCached: bool = True) -> None:
    """
    Plays a wav file by filename

    :param filename: string - filename of the wav file to play
    :param useCached: [opt] bool - False = Dump any previously cached wav associated with filename

    @python_v14 Added new option **useCached**.

    Example::

        ..
        xbmc.playSFX('special://xbmc/scripts/dingdong.wav')
        xbmc.playSFX('special://xbmc/scripts/dingdong.wav',False)
        ..
    """
    pass


def stopSFX() -> None:
    """
    Stops wav file

    @python_v14 New function added.

    Example::

        ..
        xbmc.stopSFX()
        ..
    """
    pass


def enableNavSounds(yesNo: bool) -> None:
    """
    Enables/Disables nav sounds

    :param yesNo: bool - enable (True) or disable (False) nav sounds

    Example::

        ..
        xbmc.enableNavSounds(True)
        ..
    """
    pass


def getCondVisibility(condition: str) -> bool:
    """
    Get visibility conditions

    :param condition: string - condition to check
    :return: True (if the condition is verified) or False (otherwise)

    List of boolean conditions

    .. note::
        You can combine two (or more) of the above settings by
        using ``+`` as an AND operator, ``|`` as an OR operator, ``!``
        as a NOT operator, and ``[`` and ``]`` to bracket expressions.

    Example::

        ..
        visible = xbmc.getCondVisibility('[Control.IsVisible(41) + !Control.IsVisible(12)]')
        ..
    """
    return True


def getGlobalIdleTime() -> int:
    """
    Get the elapsed idle time in seconds.

    :return: Elapsed idle time in seconds as an integer

    Example::

        ..
        t = xbmc.getGlobalIdleTime()
        ..
    """
    return 0


def getCacheThumbName(path: str) -> str:
    """
    Get thumb cache filename.

    :param path: string - path to file
    :return: Thumb cache filename

    Example::

        ..
        thumb = xbmc.getCacheThumbName('f:\\videos\\movie.avi')
        ..
    """
    return ""


def translatePath(path: str) -> str:
    """
    Returns the translated path.

    :param path: string - Path to format
    :return: Translated path

    .. note::
        Only useful if you are coding for both Linux and Windows. e.g.
        Converts 'special://home' -> '/home/[username]/.kodi' on Linux.

    @python_v19 Deprecated **xbmc.translatePath**. Moved to **xbmcvfs.translatePath**

    Example::

        ..
        fpath = xbmc.translatePath('special://home')
        ..
    """
    return ""


def getCleanMovieTitle(path: str,
                       usefoldername: bool = False) -> Tuple[str, str]:
    """
    Get clean movie title and year string if available.

    :param path: string - String to clean
    :param usefoldername: [opt] bool - use folder names (defaults to false)
    :return: Clean movie title and year string if available.

    Example::

        ..
        title, year = xbmc.getCleanMovieTitle('/path/to/moviefolder/test.avi', True)
        ..
    """
    return "", ""


def getRegion(id: str) -> str:
    """
    Returns your regions setting as a string for the specified id.

    :param id: string - id of setting to return
    :return: Region setting

    .. note::
        choices are (dateshort, datelong, time, meridiem, tempunit,
        speedunit) You can use the above as keywords for arguments.

    Example::

        ..
        date_long_format = xbmc.getRegion('datelong')
        ..
    """
    return ""


def getSupportedMedia(mediaType: str) -> str:
    """
    Get the supported file types for the specific media.

    :param media: string - media type
    :return: Supported file types for the specific media as a string

    .. note::
        Media type can be (video, music, picture). The return value is a
        pipe separated string of filetypes (eg. '.mov |.avi').  You can
        use the above as keywords for arguments.

    Example::

        ..
        mTypes = xbmc.getSupportedMedia('video')
        ..
    """
    return ""


def skinHasImage(image: str) -> bool:
    """
    Check skin for presence of Image.

    :param image: string - image filename
    :return: True if the image file exists in the skin

    .. note::
        If the media resides in a subfolder include it. (eg. home - myfiles\home-myfiles2.png).
        You can use the above as keywords for arguments.

    Example::

        ..
        exists = xbmc.skinHasImage('ButtonFocusedTexture.png')
        ..
    """
    return True


def startServer(iTyp: int, bStart: bool, bWait: bool = False) -> bool:
    """
    Start or stop a server.

    :param typ: integer - use SERVER_* constants  Used format of the returned language string

    ========================= ====================================================================== 
    Value                     Description                                                            
    ========================= ====================================================================== 
    xbmc.SERVER_WEBSERVER     To control Kodi's builtin webserver                                    
    xbmc.SERVER_AIRPLAYSERVER AirPlay is a proprietary protocol stack/suite developed by Apple Inc.  
    xbmc.SERVER_JSONRPCSERVER Control JSON-RPC HTTP/TCP socket-based interface                       
    xbmc.SERVER_UPNPRENDERER  UPnP client (aka UPnP renderer)                                        
    xbmc.SERVER_UPNPSERVER    Control built-in UPnP A/V media server (UPnP-server)                   
    xbmc.SERVER_EVENTSERVER   Set eventServer part that accepts remote device input on all platforms 
    xbmc.SERVER_ZEROCONF      Control Kodi's Avahi Zeroconf                                          
    ========================= ====================================================================== 

    :param bStart: bool - start (True) or stop (False) a server
    :param bWait: [opt] bool - wait on stop before returning (not supported by all servers)
    :return: bool - True or False

    Example::

        ..
        xbmc.startServer(xbmc.SERVER_AIRPLAYSERVER, False)
        ..
    """
    return True


def audioSuspend() -> None:
    """
    Suspend Audio engine.

    Example::

        ..
        xbmc.audioSuspend()
        ..
    """
    pass


def audioResume() -> None:
    """
    Resume Audio engine.

    Example::

        ..
        xbmc.audioResume()
        ..
    """
    pass


def getUserAgent() -> str:
    """
    Returns Kodi's HTTP UserAgent string

    :return: HTTP user agent

    Example::

        ..
        xbmc.getUserAgent()
        ..

    example output: ``Kodi/17.0-ALPHA1 (X11; Linux x86_64) Ubuntu/15.10 App_Bitness/64 Version/17.0-ALPHA1-Git:2015-12-23-5770d28``
    """
    return ""


def convertLanguage(language: str, format: int) -> str:
    """
    Returns the given language converted to the given format as a string.

    :param language: string either as name in English, two letter code (ISO 639-1), or
        three letter code (ISO 639-2/T(B)
    :param format: format of the returned language string

    ================= ========================================================== 
    Value             Description                                                
    ================= ========================================================== 
    xbmc.ISO_639_1    Two letter code as defined in ISO 639-1                    
    xbmc.ISO_639_2    Three letter code as defined in ISO 639-2/T or ISO 639-2/B 
    xbmc.ENGLISH_NAME Full language name in English (default)                    
    ================= ========================================================== 

    :return: Converted Language string

    @python_v13 New function added.

    Example::

        ..
        language = xbmc.convertLanguage(English, xbmc.ISO_639_2)
        ..
    """
    return ""



# This file is generated from Kodi source code and post-edited
# to correct code style and docstrings formatting.
# License: GPL v.3 <https://www.gnu.org/licenses/gpl-3.0.en.html>
"""
**Kodi's DRM class.**
"""
from typing import Union, Dict

__kodistubs__ = True


class CryptoSession:
    """
    **Kodi's DRM class.**

    Offers classes and functions that allow a developer to work with DRM-protected
    contents like Widevine.

    This type of functionality is closely related to the type of DRM used and the
    service to be implemented.

    Using the `CryptoSession` constructor allow you to have access to a DRM session.
    With a DRM session you can read and write the DRM
    properties `GetPropertyString`,`SetPropertyString` and establish session keys
    with GetKeyRequest and `ProvideKeyResponse`, or resume previous session keys
    with `RestoreKeys`.

    When the session keys are established you can use these methods to perform
    various operations:`Encrypt` /`Decrypt` for data encryption / decryption,`Sign`
    /`Verify` for make or verify data-signature. Useful for example to implement
    encrypted communication between a client and the server.

    An example where such functionality is useful is the Message Security Layer
    (MSL) transmission protocol used in some VOD applications. This protocol (or
    rather framework) is used to increase the level of security in the exchange of
    messages (such as licences, manifests or other data), which defines a security
    extension / layer on top of the HTTP protocol.

    Constructor for DRM crypto session

    :param UUID: string - 16 byte UUID of the DRM system to use
    :param cipherAlgorithm: string - Algorithm used for encryption / decryption ciphers
    :param macAlgorithm: string - Algorithm used for sign / verify
    :raises RuntimeException: If the session can not be established

    @python_v18 New class added.

    Example::

        ..
        uuid_widevine = 'edef8ba9-79d6-4ace-a3c8-27dcd51d21ed'
        crypto_session = xbmcdrm.CryptoSession(uuid_widevine, 'AES/CBC/NoPadding', 'HmacSHA256')
        ..
    """
    
    def __init__(self, UUID: str,
                 cipherAlgorithm: str,
                 macAlgorithm: str) -> None:
        pass
    
    def GetKeyRequest(self, init: Union[str,  bytes,  bytearray],
                      mimeType: str,
                      offlineKey: bool,
                      optionalParameters: Dict[str,  str]) -> bytearray:
        """
        Generate a key request

        Generate a key request, used for request/response exchange between the app and a
        license server to obtain or release keys used to decrypt encrypted content.
        After the app has received the key request response from the license server, it
        should deliver to the response to the DRM instance using the
        method `ProvideKeyResponse`, to activate the keys.

        :param init: byte - Initialization bytes container-specific data, its meaning is
            interpreted based on the mime type provided in the mimeType
            parameter. It could contain, for example, the content ID, key ID
            or other data required in generating the key request.
        :param mimeType: string - Type of media which is exchanged (e.g. "application/xml",
            "video/mp4")
        :param offlineKey: bool - Specifies the type of the request. The request may be to
            acquire keys for Streaming or Offline content
        :param optionalParameters: [opt] map - Will be included in the key request message to allow a
            client application to provide additional message parameters to the
            server
        :return: byte - The opaque key request data (challenge) which is send to key server

        @python_v18 New function added.

        @python_v19 With python 3 the init param must be a bytearray instead of byte.
        """
        return bytearray()
    
    def GetPropertyString(self, name: str) -> str:
        """
        Request a system specific property value of the DRM system.

        :param name: string - Name of the property to query
        :return: Value of the requested property

        @python_v18 New function added.
        """
        return ""
    
    def ProvideKeyResponse(self, response: Union[str,  bytes,  bytearray]) -> str:
        """
        Provide a key response

        When a key response is received from the license server, must be sent to the DRM
        instance by using provideKeyResponse. See also GetKeyRequest.

        :param response: byte - Key data returned from the license server
        :return: A keySetId if the response is for an offline key requests which can be used later
            with restoreKeys, else return empty for streaming key requests.

        @python_v18 New function added.

        @python_v19 With python 3 the response argument must be a bytearray instead of byte.
        """
        return ""
    
    def RemoveKeys(self) -> None:
        """
        Removes all keys currently loaded in a session.

        @python_v18 New function added.
        """
        pass
    
    def RestoreKeys(self, keySetId: str) -> None:
        """
        Restores session keys stored during previous `ProvideKeyResponse` call.

        :param keySetId: string - Identifies the saved key set to restore. This value must
            never be null.

        @python_v18 New function added.
        """
        pass
    
    def SetPropertyString(self, name: str, value: str) -> None:
        """
        Set a system specific property value in the DRM system.

        :param name: string - Name of the property. This value must never be null.
        :param value: string - Value of the property to set. This value must never be null.

        @python_v18 New function added.
        """
        pass
    
    def Decrypt(self, cipherKeyId: Union[str,  bytes,  bytearray],
                input: Union[str,  bytes,  bytearray],
                iv: Union[str,  bytes,  bytearray]) -> bytearray:
        """
        Decrypt an encrypted data by using session keys.

        :param cipherKeyId: byte - Encryption key id (provided from a service handshake)
        :param input: byte - Cipher text to decrypt
        :param iv: byte - Initialization vector of cipher text
        :return: Decrypted input data

        @python_v18 New function added.

        @python_v19 With python 3 all arguments need to be of type bytearray instead of byte.
        """
        return bytearray()
    
    def Encrypt(self, cipherKeyId: Union[str,  bytes,  bytearray],
                input: Union[str,  bytes,  bytearray],
                iv: Union[str,  bytes,  bytearray]) -> bytearray:
        """
        Encrypt data by using session keys.

        :param cipherKeyId: byte - Encryption key id (provided from a service handshake)
        :param input: byte - Encrypted text
        :param iv: byte - Initialization vector of encrypted text
        :return: byte - Encrypted input data

        @python_v18 New function added.

        @python_v19 With python 3 all arguments need to be of type bytearray instead of byte.
        """
        return bytearray()
    
    def Sign(self, macKeyId: Union[str,  bytes,  bytearray],
             message: Union[str,  bytes,  bytearray]) -> bytearray:
        """
        Generate a DRM encrypted signature for a text message.

        :param macKeyId: byte - HMAC key id (provided from a service handshake)
        :param message: byte - Message text on which to base the signature
        :return: byte - Signature

        @python_v18 New function added.

        @python_v19 With python 3 all arguments need to be of type bytearray instead of byte.
        """
        return bytearray()
    
    def Verify(self, macKeyId: Union[str,  bytes,  bytearray],
               message: Union[str,  bytes,  bytearray],
               signature: Union[str,  bytes,  bytearray]) -> bool:
        """
        Verify the validity of a DRM signature of a text message.

        :param macKeyId: byte - HMAC key id (provided from a service handshake)
        :param message: byte - Message text on which the signature is based
        :param signature: byte - The signature to verify
        :return: true when the signature is valid

        @python_v18 New function added.

        @python_v19 With python 3 for all arguments is needed to pass bytearray instead of byte.
        """
        return True



# This file is generated from Kodi source code and post-edited
# to correct code style and docstrings formatting.
# License: GPL v.3 <https://www.gnu.org/licenses/gpl-3.0.en.html>
"""
**Virtual file system functions on Kodi.**

Offers classes and functions offers access to the Virtual File Server (VFS)
which you can use to manipulate files and folders.
"""
from typing import Union, List, Tuple, Optional

__kodistubs__ = True


class File:
    """
    **Kodi's file class.**

    :param filepath: string Selected file path
    :param mode: [opt] string Additional mode options (if no mode is supplied, the
        default is Open for Read).

    ==== ============== 
    Mode Description    
    ==== ============== 
    w    Open for write 
    ==== ============== 

    @python_v19 Added context manager support

    Example::

        ..
        f = xbmcvfs.File(file, 'w')
        ..

    Example (v19 and up)::

        ..
        with xbmcvfs.File(file, 'w') as f:
            ..
            ..
    """
    
    def __init__(self, filepath: str, mode: Optional[str] = None) -> None:
        pass

    def __enter__(self):  # Required for context manager
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):  # Required for context manager
        pass

    def read(self, numBytes: int = 0) -> str:
        """
        Read file parts as string.

        :param bytes: [opt] How many bytes to read - if not set it will read the whole file
        :return: string

        Example::

            ..
            f = xbmcvfs.File(file)
            b = f.read()
            f.close()
            ..

        Example (v19 and up)::

            ..
            with xbmcvfs.File(file) as file:
                b = f.read()
                ..
        """
        return ""
    
    def readBytes(self, numBytes: int = 0) -> bytearray:
        """
        Read bytes from file.

        :param numbytes: How many bytes to read [opt]- if not set it will read the whole file
        :return: bytearray

        Example::

            ..
            f = xbmcvfs.File(file)
            b = f.readBytes()
            f.close()
            ..

        Example (v19 and up)::

            ..
            with xbmcvfs.File(file) as f:
                b = f.readBytes()
                ..
        """
        return bytearray()
    
    def write(self, buffer: Union[str,  bytes,  bytearray]) -> bool:
        """
        To write given data in file.

        :param buffer: Buffer to write to file
        :return: True on success.

        Example::

            ..
            f = xbmcvfs.File(file, 'w')
            result = f.write(buffer)
            f.close()
            ..

        Example (v19 and up)::

            ..
            with xbmcvfs.File(file, 'w') as f:
                result = f.write(buffer)
                ..
        """
        return True
    
    def size(self) -> int:
        """
        Get the file size.

        :return: The file size

        Example::

            ..
            f = xbmcvfs.File(file)
            s = f.size()
            f.close()
            ..

        Example (v19 and up)::

            ..
            with xbmcvfs.File(file) as f:
                s = f.size()
                ..
        """
        return 0
    
    def seek(self, seekBytes: int, iWhence: int = 0) -> int:
        """
        Seek to position in file.

        :param seekBytes: position in the file
        :param iWhence: [opt] where in a file to seek from[0 beginning, 1 current , 2 end
            position]

        @python_v19 Function changed. param **iWhence** is now optional.

        Example::

            ..
            f = xbmcvfs.File(file)
            result = f.seek(8129, 0)
            f.close()
            ..

        Example (v19 and up)::

            ..
            with xbmcvfs.File(file) as f:
                result = f.seek(8129, 0)
                ..
        """
        return 0
    
    def tell(self) -> int:
        """
        Get the current position in the file.

        :return: The file position

        @python_v19 New function added

        Example::

            ..
            f = xbmcvfs.File(file)
            s = f.tell()
            f.close()
            ..

        Example (v19 and up)::

            ..
            with xbmcvfs.File(file) as f:
                s = f.tell()
                ..
        """
        return 0
    
    def close(self) -> None:
        """
        Close opened file.

        Example::

            ..
            f = xbmcvfs.File(file)
            f.close()
            ..

        Example (v19 and up)::

            ..
            with xbmcvfs.File(file) as f:
                ..
                ..
        """
        pass
    

class Stat:
    """
    **Get file or file system status.**

    These class return information about a file. Execute (search) permission is
    required on all of the directories in path that lead to the file.

    :param path: [string] file or folder

    @python_v12 New function added

    Example::

        ..
        st = xbmcvfs.Stat(path)
        modified = st.st_mtime()
        ..
    """
    
    def __init__(self, path: str) -> None:
        pass
    
    def st_mode(self) -> int:
        """
        To get file protection.

        :return: st_mode
        """
        return 0
    
    def st_ino(self) -> int:
        """
        To get inode number.

        :return: st_ino
        """
        return 0
    
    def st_dev(self) -> int:
        """
        To get ID of device containing file.

        The st_dev field describes the device on which this file resides.

        :return: st_dev
        """
        return 0
    
    def st_nlink(self) -> int:
        """
        To get number of hard links.

        :return: st_nlink
        """
        return 0
    
    def st_uid(self) -> int:
        """
        To get user ID of owner.

        :return: st_uid
        """
        return 0
    
    def st_gid(self) -> int:
        """
        To get group ID of owner.

        :return: st_gid
        """
        return 0
    
    def st_size(self) -> int:
        """
        To get total size, in bytes.

        The st_size field gives the size of the file (if it is a regular file or a
        symbolic link) in bytes. The size of a symbolic link (only on Linux and Mac OS X)
        is the length of the pathname it contains, without a terminating null byte.

        :return: st_size
        """
        return 0
    
    def st_atime(self) -> int:
        """
        To get time of last access.

        :return: st_atime
        """
        return 0
    
    def st_mtime(self) -> int:
        """
        To get time of last modification.

        :return: st_mtime
        """
        return 0
    
    def st_ctime(self) -> int:
        """
        To get time of last status change.

        :return: st_ctime
        """
        return 0
    

def copy(strSource: str, strDestination: str) -> bool:
    """
    Copy file to destination, returns true/false.

    :param source: file to copy.
    :param destination: destination file
    :return: True if successed

    Example::

        ..
        success = xbmcvfs.copy(source, destination)
        ..
    """
    return True


def delete(file: str) -> bool:
    """
    Delete a file

    :param file: File to delete
    :return: True if successed

    Example::

        ..
        xbmcvfs.delete(file)
        ..
    """
    return True


def rename(file: str, newFile: str) -> bool:
    """
    Rename a file

    :param file: File to rename
    :param newFileName: New filename, including the full path
    :return: True if successed

    .. note::
        Moving files between different filesystem (eg. local to nfs://) is
        not possible on all platforms. You may have to do it manually by
        using the copy and deleteFile functions.

    Example::

        ..
        success = xbmcvfs.rename(file,newFileName)
        ..
    """
    return True


def exists(path: str) -> bool:
    """
    Check for a file or folder existence

    :param path: File or folder (folder must end with slash or backslash)
    :return: True if successed

    Example::

        ..
        success = xbmcvfs.exists(path)
        ..
    """
    return True


def makeLegalFilename(filename: str) -> str:
    """
    Returns a legal filename or path as a string.

    :param filename: string - filename/path to make legal
    :return: Legal filename or path as a string

    .. note::
        The returned value is platform-specific. This is due to the fact
        that the chars that need to be replaced to make a path legal
        depend on the underlying OS filesystem. This is useful, for
        example, if you want to create a file or folder based on data over
        which you have no control (e.g. an external API).

    @python_v19 New function added (replaces old **xbmc.makeLegalFilename**)

    Example::

        ..
        # windows
        >> xbmcvfs.makeLegalFilename('C://Trailers/Ice Age: The Meltdown.avi')
        C:\Trailers\Ice Age_ The Meltdown.avi
        # non-windows
        >> xbmcvfs.makeLegalFilename("///\\jk???lj????.mpg")
        /jk___lj____.mpg
        ..
    """
    return ""


def translatePath(path: str) -> str:
    """
    Returns the translated path.

    :param path: string - Path to format
    :return: Translated path

    .. note::
        Only useful if you are coding for both Linux and Windows. e.g.
        Converts 'special://home' -> '/home/[username]/.kodi' on Linux.

    @python_v19 New function added (replaces old **xbmc.translatePath**)

    Example::

        ..
        fpath = xbmcvfs.translatePath('special://home')
        ..
    """
    return ""


def validatePath(path: str) -> str:
    """
    Returns the validated path.

    :param path: string - Path to format
    :return: Validated path

    .. note::
        The result is platform-specific. Only useful if you are coding for
        multiple platfforms for fixing slash problems (e.g. Corrects
        'Z://something' -> 'Z:\something').

    @python_v19 New function added (replaces old **xbmc.validatePath**)

    Example::

        ..
        fpath = xbmcvfs.validatePath(somepath)
        ..
    """
    return ""


def mkdir(path: str) -> bool:
    """
    Create a folder.

    :param path: Folder to create
    :return: True if successed

    Example::

        ..
        success = xbmcvfs.mkdir(path)
        ..
    """
    return True


def mkdirs(path: str) -> bool:
    """
    Make all directories along the path

    Create folder(s) - it will create all folders in the path.

    :param path: Folders to create
    :return: True if successed

    Example::

        ..
        success = xbmcvfs.mkdirs(path)
        ..
    """
    return True


def rmdir(path: str, force: bool = False) -> bool:
    """
    Remove a folder.

    :param path: Folder to remove
    :return: True if successed

    Example::

        ..
        success = xbmcvfs.rmdir(path)
        ..
    """
    return True


def listdir(path: str) -> Tuple[List[str], List[str]]:
    """
    Lists content of a folder.

    :param path: Folder to get list from
    :return: Directory content list

    Example::

        ..
        dirs, files = xbmcvfs.listdir(path)
        ..
    """
    return [""], [""]



# coding: utf-8

from io import open
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
from kodistubs_meta import AUTHOR, EMAIL, VERSION

with open('README.rst', encoding='utf-8') as fo:
    long_descr = fo.read()

setup(
    name='Kodistubs',
    version=VERSION,
    py_modules=[
        'xbmc', 'xbmcaddon', 'xbmcgui', 'xbmcplugin', 'xbmcvfs', 'xbmcdrm'
    ],
    zip_safe=False,
    python_requires='>=3.6',
    description='Stub modules that re-create Kodi Python API',
    long_description=long_descr,
    author=AUTHOR,  # The new Kodistubs have been generated from scratch
    author_email=EMAIL,
    url='https://github.com/romanvm/Kodistubs',
    license='GPLv3',
    keywords="kodi documentation inspection",
    classifiers=[
        'Environment :: Plugins',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ]
)



# This file is generated from Kodi source code and post-edited
# to correct code style and docstrings formatting.
# License: GPL v.3 <https://www.gnu.org/licenses/gpl-3.0.en.html>
"""
**Plugin functions on Kodi.**

Offers classes and functions that allow a developer to present information
through Kodi's standard menu structure. While plugins don't have the same
flexibility as scripts, they boast significantly quicker development time and a
more consistent user experience.
"""
from typing import List, Tuple, Optional

from xbmcgui import ListItem

__kodistubs__ = True

SORT_METHOD_ALBUM = 14
SORT_METHOD_ALBUM_IGNORE_THE = 15
SORT_METHOD_ARTIST = 11
SORT_METHOD_ARTIST_IGNORE_THE = 13
SORT_METHOD_BITRATE = 43
SORT_METHOD_CHANNEL = 41
SORT_METHOD_COUNTRY = 17
SORT_METHOD_DATE = 3
SORT_METHOD_DATEADDED = 21
SORT_METHOD_DATE_TAKEN = 44
SORT_METHOD_DRIVE_TYPE = 6
SORT_METHOD_DURATION = 8
SORT_METHOD_EPISODE = 24
SORT_METHOD_FILE = 5
SORT_METHOD_FULLPATH = 35
SORT_METHOD_GENRE = 16
SORT_METHOD_LABEL = 1
SORT_METHOD_LABEL_IGNORE_FOLDERS = 36
SORT_METHOD_LABEL_IGNORE_THE = 2
SORT_METHOD_LASTPLAYED = 37
SORT_METHOD_LISTENERS = 39
SORT_METHOD_MPAA_RATING = 31
SORT_METHOD_NONE = 0
SORT_METHOD_PLAYCOUNT = 38
SORT_METHOD_PLAYLIST_ORDER = 23
SORT_METHOD_PRODUCTIONCODE = 28
SORT_METHOD_PROGRAM_COUNT = 22
SORT_METHOD_SIZE = 4
SORT_METHOD_SONG_RATING = 29
SORT_METHOD_SONG_USER_RATING = 30
SORT_METHOD_STUDIO = 33
SORT_METHOD_STUDIO_IGNORE_THE = 34
SORT_METHOD_TITLE = 9
SORT_METHOD_TITLE_IGNORE_THE = 10
SORT_METHOD_TRACKNUM = 7
SORT_METHOD_UNSORTED = 40
SORT_METHOD_VIDEO_RATING = 19
SORT_METHOD_VIDEO_RUNTIME = 32
SORT_METHOD_VIDEO_SORT_TITLE = 26
SORT_METHOD_VIDEO_SORT_TITLE_IGNORE_THE = 27
SORT_METHOD_VIDEO_TITLE = 25
SORT_METHOD_VIDEO_USER_RATING = 20
SORT_METHOD_VIDEO_YEAR = 18


def addDirectoryItem(handle: int,
                     url: str,
                     listitem: ListItem,
                     isFolder: bool = False,
                     totalItems: int = 0) -> bool:
    """
    Callback function to pass directory contents back to Kodi.

    :param handle: integer - handle the plugin was started with.
    :param url: string - url of the entry. would be ``plugin://`` for another virtual directory
    :param listitem: ListItem - item to add.
    :param isFolder: [opt] bool - True=folder / False=not a folder(default).
    :param totalItems: [opt] integer - total number of items that will be passed.(used for
        progressbar)
    :return: Returns a bool for successful completion.

    .. note::
        You can use the above as keywords for arguments and skip certain
        optional arguments. Once you use a keyword, all following
        arguments require the keyword.

    Example::

        ..
        if not xbmcplugin.addDirectoryItem(int(sys.argv[1]), 'F:\\Trailers\\300.mov', listitem, totalItems=50): break
        ..
    """
    return True


def addDirectoryItems(handle: int,
                      items: List[Tuple[str,  ListItem,  bool]],
                      totalItems: int = 0) -> bool:
    """
    Callback function to pass directory contents back to Kodi as a list.

    :param handle: integer - handle the plugin was started with.
    :param items: List - list of (url, listitem[, isFolder]) as a tuple to add.
    :param totalItems: [opt] integer - total number of items that will be passed.(used for
        progressbar)
    :return: Returns a bool for successful completion.

    Large lists benefit over using the standard `addDirectoryItem()`. You may call
    this more than once to add items in chunks.

    Example::

        ..
        if not xbmcplugin.addDirectoryItems(int(sys.argv[1]), [(url, listitem, False,)]: raise
        ..
    """
    return True


def endOfDirectory(handle: int,
                   succeeded: bool = True,
                   updateListing: bool = False,
                   cacheToDisc: bool = True) -> None:
    """
    Callback function to tell Kodi that the end of the directory listing in a
    virtualPythonFolder module is reached.

    :param handle: integer - handle the plugin was started with.
    :param succeeded: [opt] bool - True=script completed successfully(Default),
        False=Script did not.
    :param updateListing: [opt] bool - True=this folder should update the current
        listing/False=Folder is a subfolder (Default).
    :param cacheToDisc: [opt] bool - True=Folder will cache if extended
        time(default), False=this folder will never cache to disc.

    Example::

        ..
        xbmcplugin.endOfDirectory(int(sys.argv[1]), cacheToDisc=False)
        ..
    """
    pass


def setResolvedUrl(handle: int,
                   succeeded: bool,
                   listitem: ListItem) -> None:
    """
    Callback function to tell Kodi that the file plugin has been resolved to a url

    :param handle: integer - handle the plugin was started with.
    :param succeeded: bool - True=script completed successfully, False=Script did not.
    :param listitem: ListItem - item the file plugin resolved to for playback.

    Example::

        ..
        xbmcplugin.setResolvedUrl(int(sys.argv[1]), True, listitem)
        ..
    """
    pass


def addSortMethod(handle: int,
                  sortMethod: int,
                  labelMask: str = "",
                  label2Mask: str = "") -> None:
    """
    Adds a sorting method for the media list.

    :param handle: integer - handle the plugin was started with.
    :param sortMethod: integer - see available sort methods at the bottom (or see SortUtils).

    =================================================================== ============================
    Value                                                               Description                                             
    =================================================================== ============================
    xbmcplugin.SORT_METHOD_NONE                                         Do not sort                                             
    xbmcplugin.SORT_METHOD_LABEL                                        Sort by label                                           
    xbmcplugin.SORT_METHOD_LABEL_IGNORE_THE                             Sort by the label and ignore
                                                                        "The" before
    xbmcplugin.SORT_METHOD_DATE                                         Sort by the date                                        
    xbmcplugin.SORT_METHOD_SIZE                                         Sort by the size                                        
    xbmcplugin.SORT_METHOD_FILE                                         Sort by the file                                        
    xbmcplugin.SORT_METHOD_DRIVE_TYPE                                   Sort by the drive type                                  
    xbmcplugin.SORT_METHOD_TRACKNUM                                     Sort by the track number                                
    xbmcplugin.SORT_METHOD_DURATION                                     Sort by the duration                                    
    xbmcplugin.SORT_METHOD_TITLE                                        Sort by the title                                       
    xbmcplugin.SORT_METHOD_TITLE_IGNORE_THE                             Sort by the title and ignore
                                                                        "The" before
    xbmcplugin.SORT_METHOD_ARTIST                                       Sort by the artist                                      
    xbmcplugin.SORT_METHOD_ARTIST_IGNORE_THE                            Sort by the artist and ignore
                                                                        "The" before
    xbmcplugin.SORT_METHOD_ALBUM                                        Sort by the album                                       
    xbmcplugin.SORT_METHOD_ALBUM_IGNORE_THE                             Sort by the album and ignore
                                                                        "The" before
    xbmcplugin.SORT_METHOD_GENRE                                        Sort by the genre                                       
    xbmcplugin.SORT_SORT_METHOD_VIDEO_YEAR, xbmcplugin.SORT_METHOD_YEAR Sort by the year                                        
    xbmcplugin.SORT_METHOD_VIDEO_RATING                                 Sort by the video rating                                
    xbmcplugin.SORT_METHOD_PROGRAM_COUNT                                Sort by the program count                               
    xbmcplugin.SORT_METHOD_PLAYLIST_ORDER                               Sort by the playlist order                              
    xbmcplugin.SORT_METHOD_EPISODE                                      Sort by the episode                                     
    xbmcplugin.SORT_METHOD_VIDEO_TITLE                                  Sort by the video title                                 
    xbmcplugin.SORT_METHOD_VIDEO_SORT_TITLE                             Sort by the video sort title                            
    xbmcplugin.SORT_METHOD_VIDEO_SORT_TITLE_IGNORE_THE                  Sort by the video sort title
                                                                        and ignore "The" before
    xbmcplugin.SORT_METHOD_PRODUCTIONCODE                               Sort by the production code                             
    xbmcplugin.SORT_METHOD_SONG_RATING                                  Sort by the song rating                                 
    xbmcplugin.SORT_METHOD_MPAA_RATING                                  Sort by the mpaa rating                                 
    xbmcplugin.SORT_METHOD_VIDEO_RUNTIME                                Sort by video runtime                                   
    xbmcplugin.SORT_METHOD_STUDIO                                       Sort by the studio                                      
    xbmcplugin.SORT_METHOD_STUDIO_IGNORE_THE                            Sort by the studio and
                                                                        ignore "The" before
    xbmcplugin.SORT_METHOD_UNSORTED                                     Use list not sorted                                     
    xbmcplugin.SORT_METHOD_BITRATE                                      Sort by the bitrate                                     
    xbmcplugin.SORT_METHOD_LISTENERS                                    Sort by the listeners                                   
    xbmcplugin.SORT_METHOD_COUNTRY                                      Sort by the country                                     
    xbmcplugin.SORT_METHOD_DATEADDED                                    Sort by the added date                                  
    xbmcplugin.SORT_METHOD_FULLPATH                                     Sort by the full path name                              
    xbmcplugin.SORT_METHOD_LABEL_IGNORE_FOLDERS                         Sort by the label names and
                                                                        ignore related folder names
    xbmcplugin.SORT_METHOD_LASTPLAYED                                   Sort by last played date                                
    xbmcplugin.SORT_METHOD_PLAYCOUNT                                    Sort by the play count                                  
    xbmcplugin.SORT_METHOD_CHANNEL                                      Sort by the channel                                     
    xbmcplugin.SORT_METHOD_DATE_TAKEN                                   Sort by the taken date                                  
    xbmcplugin.SORT_METHOD_VIDEO_USER_RATING                            Sort by the rating of
                                                                        the user of video
    xbmcplugin.SORT_METHOD_SONG_USER_RATING                             Sort by the rating of
                                                                        the user of song
    =================================================================== ============================

    :param labelMask: [opt] string - the label mask to use for the first label.  applies to:

    ========================== ========================
    sortMethod                 labelMask               
    ========================== ========================
    SORT_METHOD_TRACKNUM       Defaults to ``[%N. ]%T``
    SORT_METHOD_EPISODE        Defaults to ``%H. %T``
    SORT_METHOD_PRODUCTIONCODE Defaults to ``%H. %T``
    All other sort methods     Defaults to ``%T``
    ========================== ========================

    :param label2Mask: [opt] string - the label mask to use for the second label. Defaults
        to``%D``  applies to:

    ================================ ==================== ======================================= 
                                                                                                  
    ================================ ==================== ======================================= 
    SORT_METHOD_NONE                 SORT_METHOD_UNSORTED SORT_METHOD_VIDEO_TITLE                 
    SORT_METHOD_TRACKNUM             SORT_METHOD_FILE     SORT_METHOD_TITLE                       
    SORT_METHOD_TITLE_IGNORE_THE     SORT_METHOD_LABEL    SORT_METHOD_LABEL_IGNORE_THE            
    SORT_METHOD_VIDEO_SORT_TITLE     SORT_METHOD_FULLPATH SORT_METHOD_VIDEO_SORT_TITLE_IGNORE_THE 
    SORT_METHOD_LABEL_IGNORE_FOLDERS SORT_METHOD_CHANNEL                                          
    ================================ ==================== ======================================= 

    .. note::
        to add multiple sort methods just call the method multiple times.

    @python_v13 Added new sort **SORT_METHOD_DATE_TAKEN**, **SORT_METHOD_COUNTRY**,
    **SORT_METHOD_DATEADDED**, **SORT_METHOD_FULLPATH**, **SORT_METHOD_LABEL_IGNORE_FOLDERS**,
    **SORT_METHOD_LASTPLAYED**, **SORT_METHOD_PLAYCOUNT**, **SORT_METHOD_CHANNEL**.

    @python_v17 Added new sort **SORT_METHOD_VIDEO_USER_RATING**.

    @python_v19 Added new option **labelMask**.

    Example::

        ..
        xbmcplugin.addSortMethod(int(sys.argv[1]), xbmcplugin.SORTMETHOD_DATEADDED)
        ..
    """
    pass


def getSetting(handle: int, id: str) -> str:
    """
    Returns the value of a setting as a string.

    :param handle: integer - handle the plugin was started with.
    :param id: string - id of the setting that the module needs to access.
    :return: Setting value as string

    .. note::
        You can use the above as a keyword.

    Example::

        ..
        apikey = xbmcplugin.getSetting(int(sys.argv[1]), 'apikey')
        ..
    """
    return ""


def setSetting(handle: int, id: str, value: str) -> None:
    """
    Sets a plugin setting for the current running plugin.

    :param handle: integer - handle the plugin was started with.
    :param id: string - id of the setting that the module needs to access.
    :param value: string or unicode - value of the setting.

    Example::

        ..
        xbmcplugin.setSetting(int(sys.argv[1]), id='username', value='teamxbmc')
        ..
    """
    pass


def setContent(handle: int, content: str) -> None:
    """
    Sets the plugins content.

    :param handle: integer - handle the plugin was started with.
    :param content: string - content type (eg. movies)

    Available content strings

    ====== ======= ======== ===========
    files  songs   artists  albums      
    movies tvshows episodes musicvideos 
    videos images  games                
    ====== ======= ======== =========== 

    Use **videos** for all videos which do not apply to the more specific mentioned
    ones like "movies", "episodes" etc. A good example is youtube.

    @python_v18 Added new **games** content

    Example::

        ..
        xbmcplugin.setContent(int(sys.argv[1]), 'movies')
        ..
    """
    pass


def setPluginCategory(handle: int, category: str) -> None:
    """
    Sets the plugins name for skins to display.

    :param handle: integer - handle the plugin was started with.
    :param category: string or unicode - plugins sub category.

    Example::

        ..
        xbmcplugin.setPluginCategory(int(sys.argv[1]), 'Comedy')
        ..
    """
    pass


def setPluginFanart(handle: int,
                    image: Optional[str] = None,
                    color1: Optional[str] = None,
                    color2: Optional[str] = None,
                    color3: Optional[str] = None) -> None:
    """
    Sets the plugins fanart and color for skins to display.

    :param handle: integer - handle the plugin was started with.
    :param image: [opt] string - path to fanart image.
    :param color1: [opt] hexstring - color1. (e.g. '0xFFFFFFFF')
    :param color2: [opt] hexstring - color2. (e.g. '0xFFFF3300')
    :param color3: [opt] hexstring - color3. (e.g. '0xFF000000')

    Example::

        ..
        xbmcplugin.setPluginFanart(int(sys.argv[1]), 'special://home/addons/plugins/video/Apple movie trailers II/fanart.png', color2='0xFFFF3300')
        ..
    """
    pass


def setProperty(handle: int, key: str, value: str) -> None:
    """
    Sets a container property for this plugin.

    :param handle: integer - handle the plugin was started with.
    :param key: string - property name.
    :param value: string or unicode - value of property.

    .. note::
        Key is NOT case sensitive.

    Example::

        ..
        xbmcplugin.setProperty(int(sys.argv[1]), 'Emulator', 'M.A.M.E.')
        ..
    """
    pass
